==> nv-remote-retry-1.0.5.tgz: package/index.js <==
const _un           = undefined;
const _t            = true;
const _f            = false;
const {Buffer}      = require("buffer");
const v8            = require("v8");
const {RD} = require("nv-redis-fetch");

const creat_id_pool = require("nv-facutil-simple-pairid");

class RedisCfg {
     password='redis';
     db =15;
     port=6379;
     host='127.0.0.1';
     family=4
}



const _JSN_ENCD     = 0;           
const _V8SER_ENCD   = 1;
const _s2j          = JSON.parse;
const _j2s          = JSON.stringify;

const _decd_rv      = (rv_str)=> {
     let  rv       = s2j(rv_str);
     //
     let  [vtype,arg_ary,max_retry_times,intv] = rv;
     if(vtype === _JSN_ENCD) {
         return([vtype,arg_ary,max_retry_times,intv,_un])      
     } else {
         let buf_str = arg_ary;
         let buf  = Buffer.from(arg_ary,'latin1');
         arg_ary  = v8.deserialize(buf);
         return([vtype,arg_ary,max_retry_times,intv,buf_str])
     }
}
const _updt_rv    = (vtype,arg_ary,max_retry_times,intv,buf_str)=> {
      let nrv;
      if(vtype === _JSN_ENCD) {
           nrv = [vtype,arg_ary,max_retry_times,intv]
      } else {
           nrv = [vtype,buf_str,max_retry_times,intv]
      }
      let nrv_str = _j2s(nrv);
      return(nrv_str) 
}
const _encd_rv  =   (arg_ary,max_retry_times,intv) => {
     let nrv = [_JSN_ENCD,arg_ary,max_retry_times,intv];
     let nrv_str = '';
     try {
         nrv_str = _j2s(nrv)
     } catch(err) {
         let buf_str = v8.serialize(arg_ary);
         nrv[0] = _V8SER_ENCD;
         nrv[1] = buf_str
     }
     return(nrv_str)
}

class RRetry extends RD {
    handle_pool = {};        // id : [handle,rid]   id:expire rid real
    handle_wrap = null;
    #get_nid    = creat_id_pool(2**48-1);
    constructor(rdcfg=new RedisCfg()) {
        super(rdcfg.password,rdcfg.db,rdcfg.port,rdcfg.host,rdcfg.family);
        ////----注册expire noti
        let self = this;
        this.handle_wrap = async function(id) {
             let  pool                           = self.handle_pool;
             if(pool[id]) {
                 let  [handle,rid]                = pool[id];
                 let  rv_str                      = await self.rd_.get(rid);
                 let [vtype,arg_ary,max_retry_times,intv,buf_str] = _decd_rv(rv_str);              
                 let [succ,R] = await handle(...arg_ary);
                 max_retry_times = max_retry_times -1;
                 if((max_retry_times ===0) || (succ===_t)) {
                      delete pool[id];              //删除handle 
                      await self.rd_.del(rid);      //删除rv
                 } else {
                      let nrv_str = _updt_rv(vtype,arg_ary,max_retry_times,intv,buf_str);
                      await self.rd_.set(rid,nrv_str);  //更新 max_retry_times
                      await self.rd_.set(id,null);
                      await self.rd_.pexpire(id,intv);
                 }
                 return([succ,R])
             } else {
                 //任务已经被cancel
                 return(null)
             }
        };
    }
    async init() {
        await this.regis_expired_noti(this.handle_wrap);
    }
    async add_retry_task(handle,arg_ary,max_retry_times=1,intv=60000) {
         let [id,rid]  = this.#get_nid();
         let rv_str    = _encd_rv(arg_ary,max_retry_times,intv); 
         await this.rd_.set(rid,rv_str);
         await this.rd_.set(id,null);
         await this.rd_.pexpire(id,intv);            //启动
         return([id,rid])
    }
    async cancel_retry_task(id) {
         let pool         = this.handle_pool;
         if(pool[id]) {
             let [handle,rid] = pool[id];
             delete pool[id];
             await this.rd_.del(rid);
             // id 让 redis 自动删除
         } else { 
         }
    }
    async list_all() {
        //debug AND test only , very slow
        let arr;
        for(let id in this.handle_pool) {
            let [handle,rid] = this.handle_pool[id];
            let  rv_str                      = await self.rd_.get(rid);
            if(rv_str!== null) {
                let [vtype,arg_ary,max_retry_times,intv,buf_str] = _decd_rv(rv_str);       
                arr.push(arg_ary);
            } else {
            }
        }
        return(arr)
    }
}

module.exports = RRetry;

==> nv-remote-retry-1.0.7.tgz: package/index.js <==
const _un           = undefined;
const _nu           = null;
const _t            = true;
const _f            = false;
const {Buffer}      = require("buffer");
const v8            = require("v8");
const {RD} = require("nv-redis-fetch");

const creat_id_pool = require("nv-facutil-simple-pairid");

class RedisCfg {
     password='redis';
     db =15;
     port=6379;
     host='127.0.0.1';
     family=4
}



const _JSN_ENCD     = 0;           
const _V8SER_ENCD   = 1;
const _s2j          = JSON.parse;
const _j2s          = JSON.stringify;

const _decd_rv      = (rv_str)=> {
     let  rv       = s2j(rv_str);
     //
     let  [vtype,arg_ary,max_retry_times,intv] = rv;
     if(vtype === _JSN_ENCD) {
         return([vtype,arg_ary,max_retry_times,intv,_un])      
     } else {
         let buf_str = arg_ary;
         let buf  = Buffer.from(arg_ary,'latin1');
         arg_ary  = v8.deserialize(buf);
         return([vtype,arg_ary,max_retry_times,intv,buf_str])
     }
}
const _updt_rv    = (vtype,arg_ary,max_retry_times,intv,buf_str)=> {
      let nrv;
      if(vtype === _JSN_ENCD) {
           nrv = [vtype,arg_ary,max_retry_times,intv]
      } else {
           nrv = [vtype,buf_str,max_retry_times,intv]
      }
      let nrv_str = _j2s(nrv);
      return(nrv_str) 
}
const _encd_rv  =   (arg_ary,max_retry_times,intv) => {
     let nrv = [_JSN_ENCD,arg_ary,max_retry_times,intv];
     let nrv_str = '';
     try {
         nrv_str = _j2s(nrv)
     } catch(err) {
         let buf_str = v8.serialize(arg_ary);
         nrv[0] = _V8SER_ENCD;
         nrv[1] = buf_str
     }
     return(nrv_str)
}

const MAX_20MIN_NEXT_INTV   = (intv) => {
    if(intv < 20*60*1000) {
         intv = intv*2
    } else {}
    return(intv)
}

class RRetry extends RD {
    handle_pool      = {};        // id : [handle,rid]   id:expire rid real
    handle_wrap      = _nu;
    #get_nid         = creat_id_pool(2**48-1);
    constructor(rdcfg=new RedisCfg()) {
        super(rdcfg.password,rdcfg.db,rdcfg.port,rdcfg.host,rdcfg.family);
        ////----注册expire noti
        let self = this;
        this.handle_wrap = async function(id) {
             let  pool                           = self.handle_pool;
             if(pool[id]) {
                 let  [handle,rid,next_intv]                = pool[id];
                 let  rv_str                      = await self.rd_.get(rid);
                 let [vtype,arg_ary,max_retry_times,intv,buf_str] = _decd_rv(rv_str);              
                 let [succ,R] = await handle(...arg_ary);
                 max_retry_times = max_retry_times -1;
                 if((max_retry_times ===0) || (succ===_t)) {
                      delete pool[id];              //删除handle 
                      await self.rd_.del(rid);      //删除rv
                 } else {
                      if(next_intv!==_un) { intv = next_intv(intv);} else {}
                      let nrv_str = _updt_rv(vtype,arg_ary,max_retry_times,intv,buf_str);
                      await self.rd_.set(rid,nrv_str);  //更新 max_retry_times
                      await self.rd_.set(id,_nu);
                      await self.rd_.pexpire(id,intv);
                 }
                 return([succ,R])
             } else {
                 //任务已经被cancel
                 return(_nu)
             }
        };
    }
    async init() {
        await this.regis_expired_noti(this.handle_wrap);
    }
    async add_retry_task(handle,arg_ary,max_retry_times=1,intv=60000,next_intv=MAX_20MIN_NEXT_INTV) {
         let [id,rid]  = this.#get_nid();
         let pool      = this.handle_pool;
         pool[id]      = [handle,rid,next_intv]
         let rv_str    = _encd_rv(arg_ary,max_retry_times,intv); 
         await this.rd_.set(rid,rv_str);
         await this.rd_.set(id,_nu);
         await this.rd_.pexpire(id,intv);            //启动
         return([id,rid])
    }
    async cancel_retry_task(id) {
         let pool         = this.handle_pool;
         if(pool[id]) {
             let [handle,rid,next_intv] = pool[id];
             delete pool[id];
             await this.rd_.del(rid);
             // id 让 redis 自动删除
         } else { 
         }
    }
    async list_all() {
        //debug AND test only , very slow
        let arr;
        for(let id in this.handle_pool) {
            let [handle,rid,next_intv] = this.handle_pool[id];
            let  rv_str                      = await self.rd_.get(rid);
            if(rv_str!== _nu) {
                let [vtype,arg_ary,max_retry_times,intv,buf_str] = _decd_rv(rv_str);       
                arr.push(arg_ary);
            } else {
            }
        }
        return(arr)
    }
}

RRetry.MAX_20MIN_NEXT_INTV = MAX_20MIN_NEXT_INTV;

module.exports = RRetry;
